import java.net.*;
import java.io.*;
import java.util.*;

import javax.xml.messaging.*;
import javax.xml.soap.*;

class ExchangeClientFault {

	public ExchangeClientFault() {
		sendMessage( "aaa", "aaa" );
	}
	
	protected void sendMessage( String divisaIniziale, String divisaFinale ) {
		try {
			MessageFactory 	mf  = MessageFactory.newInstance();
			SOAPMessage 	msg = mf.createMessage();
			SOAPPart 		sp  = msg.getSOAPPart();
			SOAPEnvelope 	env = sp.getEnvelope();
			SOAPHeader 		hdr = env.getHeader();
			SOAPBody 		bdy = env.getBody();
			
			String xsi = "http://www.w3.org/2001/XMLSchema-instance";
			env.addNamespaceDeclaration("xsi", xsi); 
			env.addNamespaceDeclaration("xsd", "http://www.w3.org/2001/XMLSchema");
			env.setEncodingStyle("http://schemas.xmlsoap.org/soap/encoding/");

			javax.xml.soap.Name xsiTypeString = env.createName("type", "xsi", xsi);
			
			SOAPBodyElement gltp
				= bdy.addBodyElement(env.createName("getRate",
				"ns1",
				"urn:xmethods-CurrencyExchange"));
	
			SOAPElement e1 =
			gltp.addChildElement(env.createName("country1")).addTextNode(divisaIniziale);
			e1.addAttribute( xsiTypeString, "xsd:string" );
			
			SOAPElement e2 =
			gltp.addChildElement(env.createName("country2")).addTextNode(divisaFinale);
			e2.addAttribute( xsiTypeString, "xsd:string" );
								
			FileOutputStream sentFile = new FileOutputStream("sent.xml");
			msg.writeTo(sentFile);
			sentFile.close();
			
			URLEndpoint endpoint = new 
				URLEndpoint(System.getProperty("endpoint"));
			
			SOAPConnectionFactory scf = SOAPConnectionFactory.newInstance();
			SOAPConnection conn = scf.createConnection();
			SOAPMessage response = conn.call( msg, endpoint );
			
			if( response != null ) {
				//L'intestazione XML viene gi messa da sola
				FileOutputStream replyFile = new FileOutputStream("reply.xml");
				response.writeTo(replyFile);
				replyFile.close();
				
				sp = response.getSOAPPart();
				env = sp.getEnvelope();
				bdy = env.getBody();
				
				SOAPFault fault = bdy.getFault();
				if( fault != null ) {
					processFault( fault );
				} else {
					printSOAPElementAndRecurse( bdy );
				}
			}
			
		} catch( Exception e ) {
			e.printStackTrace();
		}
	}
	
	protected void printSOAPElement( SOAPElement e ) {
		System.out.print( e.getElementName().getLocalName() );
		if( e.getValue() != null ) {
			System.out.print( " = " );
			System.out.println( e.getValue() );
		} else {
			System.out.println();
		}
	}
	
	protected void printSOAPElementAndRecurse( SOAPElement e ) {
		printSOAPElement( e );
		Iterator ii = e.getChildElements();
		while( ii.hasNext() ) {
			Object o = ii.next();
			if( o instanceof SOAPElement) {
				printSOAPElementAndRecurse( (SOAPElement)o );
			}
		}
	}
	
	protected void processFault( SOAPFault fault ) {
		System.out.println("FAULT: " + fault.getFaultCode() );
		
		Detail d = fault.getDetail();
		Iterator ii = d.getDetailEntries();
		while( ii.hasNext() ) {
			DetailEntry de = (DetailEntry)ii.next();
			printSOAPElementAndRecurse( de );
		}
	}
	
	public static void main( String[] args ) {
		new ExchangeClientFault();
	}
}