import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.xml.messaging.*;
import javax.xml.soap.*;

import javax.activation.*;
import javax.naming.*;

import javax.mail.internet.*;

public class DilbertClient {

    private SOAPConnection 	con;
    private JLabel		label;
    private JFrame 		frame;

    public DilbertClient() {        
        createUI();
        createSOAPConnection();
        call();
    }
    
    private void createUI() {
        frame = new JFrame("DilbertClient");
        label = new JLabel("Welcome to DilberClient (Version 0.1)");
        label.setBorder( new EmptyBorder( 10, 5, 10, 10 ) );
        frame.addWindowListener( new WindowAdapter() {
            public void windowClosing( WindowEvent e ) {
                System.exit(0);
            }
        });
        frame.getContentPane().add( label );
        frame.pack();
        frame.setVisible( true );
    }
    
    private void createSOAPConnection() {
        try {
	    SOAPConnectionFactory scf = SOAPConnectionFactory.newInstance();
            con = scf.createConnection();
        } catch(Exception e) {
            System.err.println("Unable to open a SOAPConnection");
            e.printStackTrace();
            label.setText("Error (" + e.getMessage() + ")" );
        }
    }
    
    private void call() {

        try {
            System.out.println("DilbertClient.call(): creating SOAP message");
            label.setText("Creating data");
            
            MessageFactory 	mf  = MessageFactory.newInstance();
            SOAPMessage 	msg = mf.createMessage();
            SOAPPart 		sp  = msg.getSOAPPart();
            SOAPEnvelope 	env = sp.getEnvelope();
            SOAPHeader 		hdr = env.getHeader();
            SOAPBody 		bdy = env.getBody();
            
            SOAPBodyElement gltp
                = bdy.addBodyElement(env.createName("DailyDilbertImage"));

            gltp.addChildElement(
                env.createName("DailyDilbertImageSoapIn",
                "tns",
                "http://tempuri.org/")
            ).addTextNode("Synonym");
            
            MimeHeaders mh = msg.getMimeHeaders();
            mh.setHeader( "SOAPAction", "http://tempuri.org/DailyDilbertImage" );
            msg.saveChanges();
            
            URLEndpoint urlEndpoint = new 
                URLEndpoint("http://www.esynaps.com/WebServices/DailyDiblert.asmx");
            
            FileOutputStream sentFile = new FileOutputStream("sent.xml");
            sentFile.write( "<?xml version=\"1.0\"?>".getBytes() );
            msg.writeTo(sentFile);
            sentFile.close();
            
            // Send the message to the provider using the connection.
            System.out.println("DilbertClient.call(): sending...");
            label.setText("Sending request...");

            SOAPMessage reply = con.call(msg, urlEndpoint);
            
            if (reply != null) {
            
                System.out.println("DilbertClient.call(): response received");
                label.setText("Computing response");

                sp  = reply.getSOAPPart();
             	env = sp.getEnvelope();
                bdy = env.getBody();
                
                //Ottiene DailyDilbertImageResponse
                Iterator childs = bdy.getChildElements();
                SOAPElement e = (SOAPElement)childs.next();

                //Ottiene il nodo DailyDilbertImageResult
                childs = e.getChildElements();
                e = (SOAPElement)childs.next();
                
                label.setText("Translating data");
                InputStream is = MimeUtility.decode( 
                    new ByteArrayInputStream( e.getValue().getBytes() ),
                    "base64" );
                    
                ByteArrayOutputStream imageFile = new ByteArrayOutputStream();
                byte[] buf = new byte[4096];
                while( true ) {
                    int size = is.read( buf );
                    if( size == -1 )
                        break;
                    imageFile.write( buf, 0, size );
                }
                
                label.setText("");
                label.setIcon( new ImageIcon( imageFile.toByteArray() ) );
                frame.pack();
                
                FileOutputStream replyFile = new FileOutputStream("reply.xml");
                replyFile.write( "<?xml version=\"1.0\"?>".getBytes() );
                reply.writeTo(replyFile);
                replyFile.close();
                                
            } else {
                System.err.println("No reply");
                label.setText("The server is not responding");
            }
            
        } catch(Throwable e) {
            label.setText("Error (" + e.getMessage() + ")");
            e.printStackTrace();
        }
    }
    
    public static void main(String args[]) {
        new DilbertClient();
    }
    
}

